/* -*- mode: c; c-basic-offset: 4; -*- */
#include "nds_mex_utils.h"
#include <string.h>
#include <strings.h>
#include <stdio.h>

#if !defined( MWSIZE_MAX ) && !defined(HAVE_OCTAVE)
typedef int mwSize;
#endif /* MWSIZE_MAX */

/*  Copy the contents of the specfied field of an input array to the 
 *  specified field of the output array
 */
void
copy_mxarray(const mxArray* iarray, int ifield, size_t inx, 
	     mxArray* oarray, int ofield, size_t oinx) {
    mxArray* temp = mxDuplicateArray(mxGetFieldByNumber(iarray, (int)inx, ifield));
    mxSetFieldByNumber(oarray, (int)oinx, ofield, temp);
    /* mxDestroyArray(temp); */
}

/*  Get the specified integer data item from the array.
 */
int
get_mxarray_int(const mxArray* array, int field, size_t inx) {
    int* p = (int*)mxGetData(mxGetFieldByNumber(array, (int)inx, field));
    return *p;
}

/*  Get the specified integer data item from the array.
 */
double
get_mxarray_float(const mxArray* array, int field, size_t inx) {
    const mxArray* mxa = mxGetFieldByNumber(array, (int)inx, field);
    double val;
    switch (mxGetClassID(mxa)) {
    case mxSINGLE_CLASS:
	val = *(float*)mxGetData(mxa);
	break;
    case mxDOUBLE_CLASS:
	val = *(double*)mxGetData(mxa);
	break;
    case mxINT32_CLASS:
	val = *(int*)mxGetData(mxa);
	break;
    default:
	printf("get_mxarray_float: Unrecognized class: %i field: %i\n", 
	       mxGetClassID(mxa), field);
	return 9999.;
    }
    return val;
}

/*  Parse a host IP specifier of the form "<ip-addr>:<port>"
 */
void
parse_host_name(const char* in, char** host, short* port) {
    *host = strdup(in);
    char* p = index(*host, ':');
    if (p) {
	*p++  = 0;
	*port = strtol(p, 0, 0);
    }
}

/*  Put the specified bool data into the array.
 */
int
put_mxarray_bool(mxArray* array, int field, size_t inx, int val) {
    mxArray* temp = mxCreateLogicalScalar((mxLogical)val);
    if(temp == NULL) return -1;
    mxSetFieldByNumber(array, (int)inx, field, temp);
    /* mxDestroyArray(temp); */
    return 0;
}

/*  Put the specified double data into the array.
 */
int
put_mxarray_double(mxArray* array, int field, size_t inx, double val) {
    mwSize dims = 1;
    mxArray* temp = mxCreateNumericArray(1, &dims, mxDOUBLE_CLASS, mxREAL);
    if(temp == NULL) return -1;
    *(double*)mxGetData(temp) = val;
    mxSetFieldByNumber(array, (int)inx, field, temp);
    /* mxDestroyArray(temp); */
    return 0;
}

/*  Put the specified float data into the array.
 */
int
put_mxarray_float(mxArray* array, int field, size_t inx, float val) {
    mwSize dims = 1;
    mxArray* temp = mxCreateNumericArray(1, &dims, mxSINGLE_CLASS, mxREAL);
    if(temp == NULL) return -1;
    *(float*)mxGetData(temp) = val;
    mxSetFieldByNumber(array, (int)inx, field, temp);
    /* mxDestroyArray(temp); */
    return 0;
}

/*  Put the specified int data into the array.
 */
int
put_mxarray_int(mxArray* array, int field, size_t inx, int val) {
    mwSize dims = 1;
    mxArray* temp = mxCreateNumericArray(1, &dims, mxINT32_CLASS, mxREAL);
    if(temp == NULL) return -1;
    *(int*)mxGetData(temp) = val;
    mxSetFieldByNumber(array, (int)inx, field, temp);
    /* mxDestroyArray(temp); */
    return 0;
}
	
/*  Put the specified char data into the array.
 */
int
put_mxarray_str(mxArray* array, int field, size_t inx, const char* val) {
    mxArray* temp = mxCreateString(val);
    if(temp == NULL) return -1;
    mxSetFieldByNumber(array, (int)inx, field, temp);
    return 0;
}
